! Time-stamp: <fe4.fsl.noaa.gov:/home/jacobson/TM5/proj/ctemis/trunk/src/emission_co2_ff.F90: 10 Oct 2011 (Mon) 17:54:52 GMT>

! todo:
!   ff_prefix used to hold the FFInputdir *and* the filename prefix.
!     now FFInputDir can be read independently.  Check RC files for compliance.

#define TRACEBACK write (gol,'("in ",a," (",a,", line",i5,")")') rname, __FILE__, __LINE__; call goErr
#define IF_NOTOK_RETURN(action) if (status/=0) then; TRACEBACK; action; return; end if
#define IF_ERROR_RETURN(action) if (status> 0) then; TRACEBACK; action; return; end if
#include "tm5.inc"

! Compute fossil fuel emissions of CO2 in mol m-2 s-1.  Positive is source to atmosphere.

module emission_co2_ff
  
  use GO,		only: gol, goErr, goPr, goLabel
  use Dims,		only: nregions, nlon360, nlat180
  use emission_common,	only: enkf_parms, field_1x1, EmisInputDir, co2_ff
  use emission_common,	only: nmembersloc

  implicit none

  character(len=*), parameter	:: mname = 'emission_co2_ff'

  private
  character(len=2048)		:: ff_prefix
  character(len=256)		:: ff_varname

  public	:: emission_co2_ff_init, emission_co2_ff_done, emission_co2_ff_calc

  ! use this to potentially override the EmisInputDir
  character(len=2048)		:: FFInputDir
  logical			:: month_underscores
contains

  subroutine emission_co2_ff_init(status)

    use GO,		only: TrcFile, Init, Done, ReadRc
    use global_data,	only: rcfile
    use dims,		only: im,jm,nregions

    implicit none

    character(len=*), parameter	:: rname = mname//'/emission_co2_ff_init'

    integer			:: region, imr, jmr, n
    integer, intent(out)	:: status
    type(TrcFile)		:: rcF

    call Init( rcF, rcfile, status )
    IF_NOTOK_RETURN(status=1)
    call ReadRc( rcF, 'ff.input.dir', FFInputDir ,status ,default=EmisInputDir)
    IF_ERROR_RETURN(status=1)
    ff_prefix=trim(ff_prefix)//'/'//'co2ff.country_cdiac_bp.seasnam.1x1.hdf'
    ! ff_prefix already holds the default answer
    call ReadRc( rcF, 'ff_hdf.prefix', ff_prefix ,status, ff_prefix)
    IF_ERROR_RETURN(status=1)
    call ReadRc( rcF, 'ff_hdf.varname', ff_varname ,status, 'countrypattern_cdiacbptotal_seasnam')
    IF_ERROR_RETURN(status=1)
    call ReadRc( rcF, 'ff_hdf.underscoremonth', month_underscores ,status, .false.)
    IF_ERROR_RETURN(status=1)

    call Done( rcF, status )
    IF_NOTOK_RETURN(status=1)

    write (gol,'("[emission_co2_ff_init]	FF input files to be read from """,a,""",")') trim(FFInputDir)
    call goPr
    write (gol,'("[emission_co2_ff_init]	   with filename prefix: """,a,""",")') trim(ff_prefix)
    call goPr
    write (gol,'("[emission_co2_ff_init]	   and variable name:  """,a,""".")') trim(ff_varname)
    call goPr
    if (month_underscores) then
       write (gol,'("[emission_co2_ff_init]	   with trailing underscored month.")') 
       call goPr
    endif

  end subroutine emission_co2_ff_init

  subroutine emission_co2_ff_calc(parms,co2_ff,flux_updated,status)

    use dims,			only : idate, newmonth, sec_year, dxy11,nlat180, newsrun, sec_month
    use MDF,			only : MDF_HDF4
    use ReadFromFile_MDF,	only : ReadFromFile
    use emission_data,		only : chardate
    use go_string,		only : goNum2str
    use chem_param,		only : xmc

    implicit none
    
    character(len=*), parameter			:: rname = mname//'/emission_co2_ff_calc'
    
    ! not actually used here since we currently do not optimize FF emissions
    type(enkf_parms), dimension(:), intent(in)	:: parms 

    type(field_1x1), dimension(:), intent(out)	:: co2_ff
    type(field_1x1)				:: emissions_read
    integer, intent(out)			:: status
    logical, intent(out)			:: flux_updated

    character(len=10)				:: cdate
    character(len=2)				:: cdate2
    real					:: year2month
    integer					:: j,imember

    flux_updated = .false.

    if(newmonth .or. newsrun) then
 
       flux_updated = .true.

       cdate=goNum2str(idate(1),'(i4.4)')
       cdate2=goNum2str(idate(2),'(i2)')

!      TODO:  are date attributes required?       
!       call ReadFromFile(trim(FFInputDir)//trim(ff_prefix)//trim(cdate),trim(ff_varname),co2_ff(1)%surf,status,sds_date=(/idate(1),idate(2),1,0,0,0/))

       if (month_underscores) then
          call ReadFromFile(fname=trim(FFInputDir)//'/'//trim(ff_prefix)//trim(cdate),ftype=MDF_HDF4,vname=trim(ff_varname)//'_'//trim(cdate2),buffer=emissions_read%surf,status=status)
          IF_NOTOK_RETURN(status=1)
       else
          call ReadFromFile(fname=trim(FFInputDir)//'/'//trim(ff_prefix)//trim(cdate),ftype=MDF_HDF4,vname=trim(ff_varname),buffer=emissions_read%surf,status=status)
          IF_NOTOK_RETURN(status=1)
       endif

       year2month = sec_month/sec_year

       do imember=1,nmembersloc

          co2_ff(imember)%surf=emissions_read%surf ! all members get same FF, that stored by ReadFromFile into co2_ff(1)

          write (gol,'("[emission_co2_ff_calc]	member ",i4,": fossil fuels emission global integral ",f12.4," PgC/yr.")') imember,sum(co2_ff(imember)%surf)
          call goPr

          co2_ff(imember)%surf=co2_ff(imember)%surf*year2month*1.0e12 ! from PgC 1x1_gridbox-1 yr-1 to kgC 1x1_gridbox-1 month-1

       end do

    end if

  end subroutine emission_co2_ff_calc

  subroutine emission_co2_ff_done(status)

    use dims,		only: nregions

    implicit none

    integer, intent(out)			:: status

    character(len=*), parameter	:: rname = mname//'/emission_co2_ff_done'

    status=0
    return

  end subroutine emission_co2_ff_done

end module emission_co2_ff
